/**
  Draw a green disk using an OpenGL shader.
  
  Dr. Orion Sky Lawlor, olawlor@acm.org, 2012-01-24 (Public Domain)
*/
#include "physics/world.h" /* physics::library and physics::object */
#include "ogl/glsl.h"

/* Just include library bodies here, for easy linking */
#include "physics/world.cpp" 
#include "physics/config.cpp"
#include "ogl/glsl.cpp"


/* A raytraced object */
class rayObject : public physics::object {
public:
	rayObject(void) 
		:physics::object(0.01)  /* <- our timestep, in seconds */
	{ }
	
	void simulate(physics::library &lib) { }
	
	void draw(physics::library &lib) {
		/* Load up shader */
		GLhandleARB prog=MAKE_PROGRAM_OBJECT3(
		/* vertex program */
			void main(void) {
			   myColor = gl_Color;
			   /* Include everything in onscreen position */
			   gl_Position=gl_ModelViewProjectionMatrix*gl_Vertex;
			   /* Don't project, but do modelview transforms */
			   location=(vec3)(gl_ModelViewMatrix*gl_Vertex);
			}
		,
		/* shared vertex/fragment code */
			varying vec4 myColor;
			varying vec3 location; // world coordinates of our pixel
		,
		/* fragment program */
			void main(void) {
				float radius=length(location);
				if (radius<1.0) { /* inside the disk: draw normal color */
					gl_FragColor=myColor; 
				}
				else /* outside the disk: draw nothing */
				{
					discard; 
					//gl_FragColor=vec4(1.0,0.0,0.0,0.0); // transparent
				}
			}
		);
		
		glUseProgramObjectARB(prog);

		/* Draw a big green disk */
		glColor4f(0.0f,1.0f,0.0f,0.8f);
		glBegin(GL_QUADS);
		glVertex3f(-1,0,-1);
		glVertex3f(-1,0,+1);
		glVertex3f(+1,0,+1);
		glVertex3f(+1,0,-1);
		glEnd();
		
		glUseProgramObjectARB(0);
	}
};


/* Called create a new simulation */
void physics_setup(physics::library &lib) {
	lib.world->add(new rayObject());
	lib.background[0]=lib.background[1]=0.6;
	lib.background[2]=1.0; // light blue
}

/* Called every frame to draw everything */
void physics_draw_frame(physics::library &lib) {
	/* Point and line smoothing needs usual alpha blending */
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	/* Draw opaque white ground plane */
	float groundz=-1.0;
	float groundsize=30;
	glEnable(GL_DEPTH_TEST);
	glColor3f(1,1,1);
	glBegin(GL_QUADS);
	glVertex3f(-groundsize,-groundsize,groundz);
	glVertex3f(+groundsize,-groundsize,groundz);
	glVertex3f(+groundsize,+groundsize,groundz);
	glVertex3f(-groundsize,+groundsize,groundz);
	glEnd();
	
	/* No depth while drawing ground grid */
	glDisable(GL_DEPTH_TEST);
	glColor4f(0.0,0.0,0.0,1.0);
	glEnable(GL_LINE_SMOOTH);
	groundz-=0.001;
	glLineWidth(2.0);
	if (camera.z>=groundz) { /* no grid if you're underneath ground */
		glBegin(GL_LINES);
		for (float grid=-groundsize;grid<=+groundsize;grid++) {
			glVertex3f(-groundsize,grid,groundz); 
			glVertex3f(+groundsize,grid,groundz); 
			glVertex3f(grid,-groundsize,groundz); 
			glVertex3f(grid,+groundsize,groundz); 
		}
		glEnd();
	}
	
	/* Do depth while drawing objects */
	glEnable(GL_DEPTH_TEST);
	lib.world->draw(lib);
	
	
}


